/*
 * JZ4740 CPM server.
 *
 * Copyright (C) 2018, 2020, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/cpm-jz4740.h>
#include <l4/devices/memory.h>

#include <l4/re/env.h>

#include <ipc/server.h>

#include "cpm_server.h"

// Virtual addresses for the CPM and LCD register blocks.

static l4_addr_t cpm_virt_base = 0, cpm_virt_base_end = 0;



// Access to peripheral memory.

static int setup_memory()
{
  if (get_memory("jz4740-cpm", &cpm_virt_base, &cpm_virt_base_end))
    return 1;

  return 0;
}



/* CPM server. */

class CPM_server : public CPM
{
  Cpm_jz4740_chip *_chip;

public:
  explicit CPM_server(Cpm_jz4740_chip *chip)
  : _chip(chip)
  {
  }

  long set_frequency(enum Clock_identifiers clock, uint32_t frequency)
  {
    _chip->set_frequency(clock, frequency);
    return L4_EOK;
  }

  long have_clock(enum Clock_identifiers clock, int *enabled)
  {
    *enabled = _chip->have_clock(clock);
    return L4_EOK;
  }

  long start_clock(enum Clock_identifiers clock)
  {
    _chip->start_clock(clock);
    return L4_EOK;
  }

  long stop_clock(enum Clock_identifiers clock)
  {
    _chip->stop_clock(clock);
    return L4_EOK;
  }
};



/* Main program. */

int main(void)
{
  if (setup_memory())
    return 1;

  /* Initialise the CPM abstraction. */

  Cpm_jz4740_chip cpm_device(cpm_virt_base, 12000000);

  /* Initialise and register a server object. */
  
  CPM_server obj(&cpm_device);
  
  /* Bind and start the IPC server loop. */
  
  if (ipc_server_loop_for(CPM, &obj, "cpm"))
    return 1;

  return 0;
}
